package com.toolcafe.gwtstore.shared.model;

import com.toolcafe.gwt.shared.model.ICopyable;
import com.toolcafe.gwt.shared.model.IStorable;

/**
 * Interface of a role played by a Person. Implementations of this interface are referenced from
 * the Person object. 
 * @author rick
 *
 */
public interface IRole extends ICopyable, IStorable {
	
	/**
	 * Get the Person who is the owner of this role
	 * @return
	 */
	Person getPerson();
	
	/**
	 * Set the Person who is the owner of this role
	 * @param person the Person to set
	 */
	void setPerson(Person parent);
	
	/**
	 * Create an IRole implementation class instance of the same type as this instance. This is used 
	 * to create a blank to which to copy the instance.
	 * @return the new IRole instance
	 */
	IRole createRole();
	
	/**
	 * Get the role type associated with this role. Note, it may be a good idea to use the same
	 * role type for all implementation classes in a class hierarchy. This allows us to detect
	 * generic objects, such as Partner and BMPartner (subclass of Partner)
	 * @return the role type
	 */
	RoleType getRoleType();

	/**
	 * Set the person's database key
	 * @param key the database key encoded as a String
	 */
	void setPersonKey(String key);
}
