package com.youvoiceit.shared.model;

import java.io.Serializable;
import java.util.ArrayList;

import com.google.code.twig.annotation.Child;
import com.google.code.twig.annotation.Entity;
import com.google.code.twig.annotation.Id;
import com.google.code.twig.annotation.Parent;
import com.toolcafe.gwt.client.exception.ToolCafeInputException;
import com.toolcafe.gwt.shared.model.ICopyable;
import com.toolcafe.gwt.shared.model.ILoadable;

/**
 * A voting district
 * @author rick
 *
 */
@Entity(allocateIdsBy=10, kind="VotingDistrict")
public class VotingDistrict implements ICopyable, ILoadable {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Component of a voting district. This is a part of a voting district. For example, New Hampshire
	 * state house of representative districts may be comprised of a single town, multiple towns,
	 * or portions of a town, known as Wards.
	 * @author rick
	 *
	 */
	public static class VotingDistrictComp implements Serializable {

		private static final long serialVersionUID = 1L;
		
		/** The city or town for a voting district. A VotingDistrict can contain multiple towns,
		 * or a town can contain multiple voting districts. */
		private String city;
		/** Ward is a name for a sub-location of a voting district */
		private String ward;
		/** Combination of location and subLocation to allow querying as an embedded object in twig */
		private String fullLocation;
		/** VotingDistrict parent */
		@Parent
		private VotingDistrict votingDistrict;

		public VotingDistrictComp() { }
		public VotingDistrictComp(VotingDistrict votingDistrict, String city, String ward) {
			this.votingDistrict = votingDistrict;
			this.city = city;
			this.ward = ward;
			this.fullLocation = city + ward;
		}
		/**
		 * Get the voting district parent of this component
		 * @return the VotingDistrict
		 */
		public VotingDistrict getVotingDistrict() {
			return votingDistrict;
		}
		/**
		 * Get the city/town
		 * @return the city
		 */
		public String getCity() {
			return city;
		}
		/**
		 * Get the ward within the city or town
		 * @return the ward
		 */
		public String getWard() {
			return ward;
		}
		/**
		 * @return the fullLocation
		 */
		public String getFullLocation() {
			return fullLocation;
		} 
		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return "VotingDistrictComp [city=" + city + ", ward=" + ward + "]";
		}
	}
	
	@Id
	private long id;
	/** The jurisdiction type: e.g. country, state, city */
	private JurisdictionType jurisdictionType;
	/** Name of the city, state, or country over which the official has jurisdiction. 
	 * For example, for state representatives or senators, the jurisdication would be the state.
	 * If it is the state, we use the name of a USState enum value. */
	private String jurisdiction;
	/** Sub-jurisdiction, e.g. a county. For example, in NH, state legislator districts are 
	 * assigned within counties, hence a hierarchy of state/county/district */
	private String subJurisdiction;
	/** District number */
	private int district;
	/** List of voting district components that comprise the voting district */
	@Child
	private ArrayList<VotingDistrictComp> components = new ArrayList<VotingDistrictComp>();
	
	public VotingDistrict() { }

	/**
	 * @param jurisdictionType the jurisdictionType to set
	 */
	public void setJurisdictionType(JurisdictionType jurisdictionType) {
		this.jurisdictionType = jurisdictionType;
	}

	/**
	 * @return the jurisdictionType
	 */
	public JurisdictionType getJurisdictionType() {
		return jurisdictionType;
	}

	/**
	 * @param jurisdiction the jurisdiction to set
	 */
	public void setJurisdiction(String jurisdiction) {
		this.jurisdiction = jurisdiction;
	}

	/**
	 * @return the jurisdiction
	 */
	public String getJurisdiction() {
		return jurisdiction;
	}

	/**
	 * @param subJurisdiction the subJurisdiction to set
	 */
	public void setSubJurisdiction(String subJurisdiction) {
		this.subJurisdiction = subJurisdiction;
	}

	/**
	 * @return the subJurisdiction
	 */
	public String getSubJurisdiction() {
		return subJurisdiction;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(getJurisdictionType());
		sb.append(": ");
		sb.append(getJurisdiction());
		sb.append("/");
		sb.append(getSubJurisdiction());
		sb.append(", dist = ");
		sb.append(getDistrict());
		sb.append(", comps = ");
		sb.append(getComponents());
		
		return sb.toString();
	}

	@Override
	public void copyFrom(Object source, boolean deepCopy)
			throws ToolCafeInputException {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void loadObject() {
		// TODO Auto-generated method stub
		
	}

	public String getId() {
		return String.valueOf(id);
	}

	public void setId(long id) {
		this.id = id;
	}

	/**
	 * The district for the elected official
	 * @param district the district to set
	 */
	public void setDistrict(int district) {
		this.district = district;
	}

	/**
	 * The district for the elected official
	 * @return the district
	 */
	public int getDistrict() {
		return district;
	}

	/**
	 * Set the voting district components
	 * @param components the components to set
	 */
	public void setComponents(ArrayList<VotingDistrictComp> components) {
		this.components = components;
	}

	/**
	 * Get the voting district components
	 * @return the components
	 */
	public ArrayList<VotingDistrictComp> getComponents() {
		return components;
	}
	
}
